#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import events
import playlist
import window

from helper import formattime, debug

class playlistwin(window.window):
    def __init__(self,
                 screen, h, w, y, x, hasscrollbar,
                 songdb, logfilename=None):
        self.songdb = songdb
        
        window.window.__init__(self,
                               screen, h, w, y, x, "Playlist", hasscrollbar)
        
        self.playlist = playlist.playlist(self, logfilename)
        
        events.ec.subscribe(events.keypressed, self.keypressed)
        events.ec.subscribe(events.activewinchanged, self.activewinchanged)


    # event handlers

    def keypressed(self, event):
        key = event.key

        if self.active and key==curses.KEY_DOWN:
            self.playlist.selectnext()
        elif self.active and key==curses.KEY_UP:
            self.playlist.selectprev()
        elif self.active and key in (ord("N")-64, curses.KEY_NPAGE):
            self.playlist.selectnextpage()
        elif self.active and key in (ord("P")-64, curses.KEY_PPAGE):
            self.playlist.selectprevpage()
        elif self.active and key in (ord("A")-64, curses.KEY_HOME):
            self.playlist.selectfirst()
        elif self.active and key in (ord("E")-64, curses.KEY_END):
            self.playlist.selectlast()
        elif self.active and key in (ord("\t"), curses.KEY_LEFT):
            events.ec.notify(events.activatefilelist())
        elif self.active and key==ord("+"):
            self.playlist.moveup()
        elif self.active and key==ord("-"):
            self.playlist.movedown()
        elif self.active and key in (ord("d"), ord("D"), curses.KEY_DC):
            self.playlist.deleteselected()
        elif key==curses.KEY_BACKSPACE:
            self.playlist.deleteplayedsongs()
        elif key==(ord("D")-64):
            self.playlist.clear()
        elif key==(ord("W")-64):
            events.ec.notify(events.requestinput("Save playlist: ",
                                                 self.saveplaylisthandler))
        elif key==(ord("R")-64):
            events.ec.notify(events.requestinput("Load playlist: ",
                                                 self.loadplaylisthandler))
        else:
            return
            
        self.update()
        raise events.TerminateEventProcessing

    def activewinchanged(self, event):
        self.playlist.active = self.active = event.active is self
        if self.active:
            events.ec.notify(events.selectionchanged(self.playlist.getselected()))
        self.update()

    def updatestatusbar(self):
        if self.playlist.selected is not None:
            statusbar = [ ("d", curses.A_BOLD),
                          (": delete item | ", curses.A_NORMAL),
                          ("+", curses.A_BOLD),
                          ("/", curses.A_NORMAL),
                          ("-", curses.A_BOLD),
                          (": move item up/down | ", curses.A_NORMAL),
                          ]
        else:
            statusbar = []

        statusbar += [ ("TAB", curses.A_BOLD),
                       (": switch to database | ", curses.A_NORMAL),
                       ("p", curses.A_BOLD),
                       (": play", curses.A_NORMAL),
                       ]

        events.ec.notify(events.updatestatusbar(statusbar))

    def saveplaylisthandler(self, name, key):
        if key==ord("\n"):
            if name[-4:]!=".m3u":
                name = name + ".m3u"
            try:
                file = open(name, "w")
                for song in self.playlist:
                    file.write("%s\n" % song.path)
                file.close()
            except IOError, OSError:
                pass
            

    def loadplaylisthandler(self, name, key):
        if key==ord("\n"):
            if name[-4:]!=".m3u":
                name = name + ".m3u"
            try:
                file = open(name, "r")
                self.playlist.clear()
            
                for line in file.xreadlines():
                    self.playlist.append(
                        playlist.playlistitem(self.songdb.getsong(line.strip())))

                file.close()
                self.update()
            except:
                pass

    def updatescrollbar(self):
        self.drawscrollbar(self.playlist.top, len(self.playlist))

    # window update method

    def update(self):
        self.settitle("Playlist (-%s/%s)" % (formattime(self.playlist.ttime-
                                                        self.playlist.ptime),
                                             formattime(self.playlist.ttime)))

        window.window.update(self)
        if self.active:
            self.updatestatusbar()
            
        for i in range(self.playlist.top, self.playlist.top+self.ih):
            attr = curses.A_NORMAL

            if i<len(self.playlist):
                song = self.playlist[i]
                name = song.title
                if not song.hasbeenplayed():
                    attr = curses.A_BOLD
                if i==self.playlist.selected and self.active:
                    attr |= curses.A_REVERSE
            else:
                name = ""
            self.win.addstr(i-self.playlist.top+1, 1,
                            name.ljust(self.iw)[:self.iw],
                            attr)


        self.updatescrollbar()

        self.refresh()
