#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import curses.ascii

import window
import events

class statusbar(window.window):
    def __init__(self, screen, line, width):
        self.content = []
        self.inputmode = 0
        window.window.__init__(self, screen, 1, width, line, 0, None)
        events.ec.subscribe(events.updatestatusbar, self.updatestatusbar)
        events.ec.subscribe(events.requestinput, self.requestinput)
        events.ec.subscribe(events.keypressed, self.keypressed)

    # event handler

    def updatestatusbar(self, event):
        self.content = event.content
        self.update()

    def requestinput(self, event):
        self.inputmode = 1
        self.inputstring = ""
        self.inputprompt = event.prompt
        self.inputhandler = event.handler
        events.ec.subscribe(events.timer, self.timer)
        self.update()

    def keypressed(self, event):
        if self.inputmode:
            key = event.key
            if curses.ascii.isalnum(key) or key==ord(" "):
                self.inputstring += chr(key)
            elif key==ord("\n"):
                self.inputmode = 0
            elif key in (27, ord("G")-64):
                self.inputmode = 0
                events.ec.unsubscribe(events.timer, self.timer)
            elif key==curses.KEY_BACKSPACE:
                self.inputstring = self.inputstring[:-1]
            self.update()
            self.inputhandler(self.inputstring, key)
            raise events.TerminateEventProcessing

    def timer(self, event):
        # we want to get the cursor position right...
        self.update()
        
    # update method
    
    def update(self):
        window.window.update(self)
        self.win.move(0,0)
        self.win.clrtoeol()
        if self.inputmode:
            try:
                curses.curs_set(1)
            except:
                pass
            self.win.addstr(self.inputprompt, curses.A_BOLD)
            self.win.addstr(self.inputstring, curses.A_NORMAL)
        else:
            try:
                curses.curs_set(0)
            except:
                pass

            for element in self.content:
                try:
                    self.win.addstr(element[0], element[1])
                except:
                    pass
        self.win.refresh()
            
